#!/bin/bash
. `dirname $0`"/DDR_shared_functions"

CPPROD_UTIL=$CPDIR/bin/cpprod_util
SU="/bin/su" 
PWD="/bin/pwd"
MAX_TRIES=60
SLEEP="/bin/sleep"
CAT="/bin/cat"
ECHO="/bin/echo"
GREP="/bin/grep"
CHMOD="/bin/chmod"
SED="/bin/sed"
PSQL_CLIENT="$MDS_FWDIR/bin/psql_client"

CPPROD_UTIL=$CPDIR/bin/cpprod_util
ECHO="/bin/echo"
# exit codes of cpm_status.sh
CPM_NOT_RUNNING=13
CPM_FAILED_START=14

# if our version is prior to R80 then this script is not relevant , return success
isVersionOlderThan "R80"
if [ $? == 0 ]; then
	$ECHO "success"
	exit 0
fi


check_postgres_status()
{
	local RET=1
	local RETURN_VAL=$1
	cd $PGDIR
	$ECHO "==================================================$FWDIR/scripts/postgres_commands.sh status" >> /opt/DDR/collect_mgmtready_output 2>&1
	$SU -s /bin/bash -c "$FWDIR/scripts/postgres_commands.sh status" cp_postgres >> /opt/DDR/collect_mgmtready_output 2>&1
	RET=$?
	eval $RETURN_VAL="'$RET'"
}

busy_wait_check_postgres_status()
{
	local RETURN_VAL=$1
	cd $PGDIR
	TRIES=0
	local POSTGRES_STATUS_RET=1
	local QUERY_RET=1
	if [ -f /var/tmp/postgres_max_tries ];then
		MAX_TRIES=`$CAT /var/tmp/postgres_max_tries`
	fi
	while [ $POSTGRES_STATUS_RET -ne 0 -a $TRIES -lt $MAX_TRIES ]
	do
		check_postgres_status POSTGRES_STATUS_RET
		$SLEEP 1
		let TRIES=TRIES+1
	done
	TRIES=0
	if [ $POSTGRES_STATUS_RET -eq 0 ];then
		TRIES=0
		while [ $QUERY_RET -ne 0 -a $TRIES -lt $MAX_TRIES ]
		do
			$ECHO "==================================================$FWDIR/bin/psql_client -U postgres cpm -c select 1" >> /opt/DDR/collect_mgmtready_output 2>&1
			$FWDIR/bin/psql_client -U postgres cpm -c "select 1;" >> /opt/DDR/collect_mgmtready_output 2>&1
			QUERY_RET=$?
			$SLEEP 1
			let TRIES=TRIES+1
		done
	else
		# create resolve script
		$ECHO "#!/bin/bash" > /opt/DDR/resolve_permissions
		$ECHO "$CHMOD 755 $FWDIR/..; $CHMOD 755 $FWDIR/; $CHMOD 755 $FWDIR/scripts/; $CHMOD 755 $FWDIR/scripts/postgres_commands.sh;  $CHMOD 755 $CPDIR/lib; $CHMOD 755 $CPDIR/lib/libcrypto.so.1.0.0; exit 0;" >> /opt/DDR/resolve_permissions
		$CHMOD 777 /opt/DDR/resolve_permissions
		# print message to user
		$ECHO "Could not run compatibility check - run '/opt/DDR/resolve_permissions' and try again." >> /opt/DDR/collect_mgmtready_output 2>&1	
	fi		
	eval $RETURN_VAL="'$QUERY_RET'"
}

isManagement=`$CPPROD_UTIL CPPROD_GetValue FW1 FWManagement 4`
isLogServer=`$CPPROD_UTIL CPPROD_GetValue FW1 LogServer 4`
STATUS=0

# if our version is prior to R80 then this script is not relevant , return success
ver=`$CPPROD_UTIL CPPROD_GetVerText CPshared`
ver=`$ECHO $ver` # remove spaces
if ! [[ "X$ver" == "XR80"* ]]; then
		$ECHO "success"
		exit 0
fi


if [ "$isManagement" == "1" -o "$isLogServer" == "1" ]; then

  # start postgres
  if [ "X$MDSDIR" != "X" ]; then
      FWDIR=$MDS_TEMPLATE
  fi
  POSTGRES_WAS_DOWN=0
  POSTGRES_STATUS=3
  
  WORK_DIR=`$PWD`
  check_postgres_status POSTGRES_STATUS
  cd $WORK_DIR
  if [ $POSTGRES_STATUS -ne 0 ]; then
	POSTGRES_WAS_DOWN=1
  fi
  if [ $POSTGRES_WAS_DOWN -eq 1 ];then
	  $ECHO "==================================================$FWDIR/scripts/postgres_start.sh" >> /opt/DDR/collect_mgmtready_output 2>&1
	  $FWDIR/scripts/postgres_start.sh  >> /opt/DDR/collect_mgmtready_output 2>&1
	  POSTGRES_STATUS=3
	  busy_wait_check_postgres_status POSTGRES_STATUS
	  cd $WORK_DIR
	  if [ $POSTGRES_STATUS -ne 0 ]; then
		  $ECHO "Failed to start postgres. See /opt/DDR/collect_mgmtready_output for more information."
		  exit 0
	  fi
  fi

  #prevent an upgrade from R80/R80.10 to R80.40 in case when Postgres DB contain UTM-1 / EDGE objects.
	#if MDS - join two tables - one for object name, the second for cma name.
	if [ "X$MDSDIR" != "X" ]; then
		objects=$(psql_client cpm postgres -t -c "select distinct FORMAT('%s (Domain: %s) ',dletable.name, domainbase_data.name) from (select name, domainid from dleobjectderef_data where dlesession=0 and deleted=false and (objclass ='com.checkpoint.objects.classes.dummy.CpmiSofawareGateway' or objclass='com.checkpoint.objects.classes.dummy.CpmiLsmGatewayProfile')) as dletable INNER JOIN domainbase_data ON(dletable.domainid = domainbase_data.objid and domainbase_data.name!='System Data' and domainbase_data.objid!='a0bbbc99-adef-4ef8-bb6d-defdefdefdef');")
	else
		objects=$(psql_client cpm postgres -t -c "select distinct FORMAT('%s ',dletable.name) from (select name, domainid from dleobjectderef_data where dlesession=0 and deleted=false and (objclass ='com.checkpoint.objects.classes.dummy.CpmiSofawareGateway' or objclass='com.checkpoint.objects.classes.dummy.CpmiLsmGatewayProfile')) as dletable INNER JOIN domainbase_data ON(dletable.domainid = domainbase_data.objid and domainbase_data.name!='System Data' and domainbase_data.objid!='a0bbbc99-adef-4ef8-bb6d-defdefdefdef');")
	fi		
		
	if [ ! -z "$objects" ]; then
		output=$($ECHO -e "UTM-1 Edge Gateways, LSM UTM-1 Edge Profiles \(in SmartConsole\) and UTM-1 Edge Gateways \(in SmartProvisioning\) must be deleted prior to proceeding with the Management upgrade process. '\n'Note: Existing UTM-1 Edge Gateways \(from SmartProvisioning\) are not printed below.")
		#count the numbers of words(name of object cannot contain spaces)
		sizeofobjects=`$ECHO "${objects}" | wc -w`
		if [ "X$MDSDIR" != "X" ]; then
			#count the numbers of "(" - (name of cma can contain spaces)
			char="("
			sizeofobjects=`$GREP -o "(" <<<"$objects" | wc -l`
		fi
		#if there are less than 10 objects, print the names
		if [ "$sizeofobjects" -le 10 ]
			then 
			output=$($ECHO -e "${output} '\n'These are the UTM-1 Edge Gateways, LSM UTM-1 Edge Profiles that currently exist on the management: \"\n${objects}\"")
				
		fi
		
	fi
	
#get cpm_status:
	$ECHO "Checking cpm status to decide if we should stop postgres" >> /opt/DDR/collect_mgmtready_output
	$MDS_FWDIR/scripts/cpm_status.sh no_print
	let cpm_status=$?   
	$ECHO "CPM status is: $cpm_status">> /opt/DDR/collect_mgmtready_output
	
# stop postgres
   if [ $POSTGRES_WAS_DOWN -eq 1 ];then
    if [ -f /etc/.wizard_accepted ]; then
	  if [ $cpm_status -eq $CPM_NOT_RUNNING ] || [ $cpm_status -eq $CPM_FAILED_START ]; then
        $ECHO "==================================================$MDS_FWDIR/scripts/postgres_stop.sh" >> /opt/DDR/collect_mgmtready_output 2>&1
        $MDS_FWDIR/scripts/postgres_stop.sh  >> /opt/DDR/collect_mgmtready_output 2>&1
	  fi
	fi	
  fi
fi


  if [ "$output" == "" ]
  then
          $ECHO "success"
  else
          $ECHO "$output"
  fi


exit 0
