#!/bin/sh

MV=/bin/mv
RM=/bin/rm
CAT=/bin/cat
GREP=/bin/grep
AWK=/bin/awk
WC=/usr/bin/wc
PS=/bin/ps
SED=/bin/sed
TELLPM=/bin/tellpm
CHMOD=/bin/chmod
ECHO=/bin/echo
LN=/bin/ln
RPM=/bin/rpm
MKDIR=/bin/mkdir
DBSET=/bin/dbset
DBGET=/bin/dbget
DASTOP=/opt/CPda/bin/dastop
CPVINFO=$CPDIR/bin/cpvinfo
LOG_FILE=/opt/CPda/bin/hook_upg.log
SLEEP=/bin/sleep
CHOWN=/bin/chown

NAVTREE=/web/xml2/navtree.xml
FEATURES=/etc/features.xml
OVERVIEW=/web/htdocs2/js/overview.js

$CAT $NAVTREE | $GREP "<name>software-updates-group</name>" 1>/dev/null 2>&1
if [ $? -eq 0 ]
then
	# Remove software updates group from navtree.xml
	$SED -i '/.*PING Start.*/,/.*PING End.*/d' $NAVTREE
fi

$CAT $NAVTREE | $GREP "<name>software-updates-group-new</name>" 1>/dev/null 2>&1
if [ $? -ne 0 ]
then
	# software updates group not found in navtree.xml - insert software updates into it
	$SED -i '/<\/application>/ i\
	<!-- DA New GUI Start -->\
	<features>\
		<name>software-updates-group-new</name>\
		<feature>\
			<name>available_updates</name>\
		</feature>\
		<feature>\
			<name>installed_updates</name>\
		</feature>\
	</features>\
	<!-- DA New GUI End -->' $NAVTREE
fi

#update the link in overview.js
$CAT $OVERVIEW | $GREP "tree/installer" 1>/dev/null 2>&1
if [ $? -eq 0 ]
then
  # replace the link
  $SED -i 's/tree\/installer/tree\/available_updates/' $OVERVIEW
fi

#update features.xml
$CAT $FEATURES | $GREP "<Name>available_updates</Name>" 1>/dev/null 2>&1
if [ $? -ne 0 ]
then
	$SED -i '/<\/Features>/ i\
	<Feature>\
		<Name>available_updates</Name>\
		<DisplayName>Updates</DisplayName>\
		<Type>config</Type>\
		<Permission>rw</Permission>\
		<WebUI>\
			<DisplayText>Available Updates</DisplayText>\
			<Description>Manage the available software packages</Description>\
			<URL>/js/available_updates.js</URL>\
			<DisplayMode>\
				<advancedURL>/js/available_updates.js</advancedURL>\
				<basicURL>/js/available_updates.js</basicURL>\
			</DisplayMode>\
			<ConfigDataCollector>/cgi-bin/installer.tcl</ConfigDataCollector>\
			<ConfigDataCollector>/cgi-bin/installer_mail.tcl</ConfigDataCollector>\
			<ConfigDataCollector>/cgi-bin/installer_policy.tcl</ConfigDataCollector>\
		</WebUI>\
		<PrefixList>\
			<PrimaryPrefix>available_updates</PrimaryPrefix>\
		</PrefixList>\
		<ClusterAware>0</ClusterAware>\
		<VRRPAware>1</VRRPAware>\
		<VsxAware>1</VsxAware>\
		<InstanceAware>0</InstanceAware>\
		<LocalUserOnly>0</LocalUserOnly>\
	</Feature>' $FEATURES
fi

$CAT $FEATURES | $GREP "<Name>installed_updates</Name>" 1>/dev/null 2>&1
if [ $? -ne 0 ]
then
	$SED -i '/<\/Features>/ i\
	<Feature>\
		<Name>installed_updates</Name>\
		<DisplayName>Updates</DisplayName>\
		<Type>config</Type>\
		<Permission>rw</Permission>\
		<WebUI>\
			<DisplayText>Current Version</DisplayText>\
			<Description>Manage the installed software packages</Description>\
			<URL>/js/installed_updates.js</URL>\
			<DisplayMode>\
				<advancedURL>/js/installed_updates.js</advancedURL>\
				<basicURL>/js/installed_updates.js</basicURL>\
			</DisplayMode>\
			<ConfigDataCollector>/cgi-bin/installer.tcl</ConfigDataCollector>\
			<ConfigDataCollector>/cgi-bin/installer_mail.tcl</ConfigDataCollector>\
			<ConfigDataCollector>/cgi-bin/installer_policy.tcl</ConfigDataCollector>\
		</WebUI>\
		<PrefixList>\
			<PrimaryPrefix>installed_updates</PrimaryPrefix>\
		</PrefixList>\
		<ClusterAware>0</ClusterAware>\
		<VRRPAware>1</VRRPAware>\
		<VsxAware>1</VsxAware>\
		<InstanceAware>0</InstanceAware>\
		<LocalUserOnly>0</LocalUserOnly>\
	</Feature>' $FEATURES
else
	$SED -i 's/Installed Updates/Current Version/g' $FEATURES
fi

$CAT $FEATURES | $GREP "<Name>software-updates-group-new</Name>" 1>/dev/null 2>&1
if [ $? -ne 0 ]
then
	# software-updates-group-new not found in features.xml - insert software-updates-group-new into it
	$SED -i '/<\/Features>/ i\
	<Feature>\
		<Name>software-updates-group-new</Name>\
		<DisplayName>Software Updates</DisplayName>\
		<Type>config</Type>\
		<Permission>rw</Permission>\
		<WebUI>\
			<DisplayText>Software Updates</DisplayText>\
			<Description>Packages updates management</Description>\
			<IconClass>su_new.png</IconClass>\
			<DisplayMode>\
				<advancedURL>/js/SU_summary.js</advancedURL>\
				<basicURL>/js/SU_summary.js</basicURL>\
			</DisplayMode>\
			<URL>/js/SU_summary.js</URL>\
		</WebUI>\
	</Feature>' $FEATURES
fi

$CHOWN :root /web/htdocs2/CPda > /dev/null 2>&1
$CHOWN :root /web/htdocs2/CPda/React > /dev/null 2>&1
$CHMOD 775 /web/htdocs2/CPda > /dev/null 2>&1

$ECHO "end of MoveToNewGUI.sh"
