#!/bin/bash
if [ "$DADIR" == "" ]; then
	DADIR="/opt/CPda"
	export DADIR
fi

UI_LOG_EVENTS_FILE=/opt/CPInstLog/ui_messages_ver2
DEBUG_LOG=/opt/CPInstLog/agent_install.log
TMP_FOLDER=/
PIGZ="/usr/bin/pigz"
if [ -x $PIGZ ]
then
  ZIP_COMMAND="--use-compress-program="$PIGZ
else
  ZIP_COMMAND="-z"
fi
GTAR_DECOMP="/bin/gtar $ZIP_COMMAND -xvf"
GTAR_LIST="/bin/gtar $ZIP_COMMAND -tf"
RPM_DECOMP="/bin/rpm -U --force"
UI_DATE_FORMAT="%a %b %d %H:%M:%S %Y"
PIDOF=/sbin/pidof
DBSET=/bin/dbset
DBGET=/bin/dbget
RM=/bin/rm

#Usage:  agent_install.sh <da_pacakge.tgz> <ns_action::Action_Src>

# clean-up stuff:
# sets DB records in order to notify CLISH
# in case of web-ui request we have to delete the uploaded file and notify the web-ui by setting poll_parameter to agent_install_completed
cleanup(){
$DBSET installer:agent_install_completed $1
$DBSET installer:agent_install_msg "$2"
if [ "$IS_WEB_UI" == "1" ]; then
    $RM $TGZ_FILE 2>&1 | tee -a $DEBUG_LOG
	
	# messaging to web-ui is according to the convention in user.cpp:
	# <notification_type> | <message-type> | date | message - html format
    [ ${PIPESTATUS[0]} != 0 ] && echo "1|0|`date +"$UI_DATE_FORMAT"`| Warning: Cannot remove TGZ file." >> $UI_LOG_EVENTS_FILE
    $DBSET installer:poll_parameter agent_install_completed
fi
}

# function to log messages to web-ui messages handler if it is web-ui request, in addition to regular logging.
# if the notification_type is 3 (success) or 0 (failure) it also terminates the execution
# notification_type is according to Notification.h
log_and_exit_if_finished() {

    echo "logging with notification_type: $1" >> $DEBUG_LOG

    # messaging to web-ui is according to the convention in user.cpp:
    # <notification_type> | <message-type> | date | message - html format
    if [ "$IS_WEB_UI" == "1" ]; then
        echo "$1|1|`date +"$UI_DATE_FORMAT"`|</b>Installation of $TGZ_FILE_BASENAME</b><br>$2<br>" >> $UI_LOG_EVENTS_FILE
    fi
    echo $2 | tee -a $DEBUG_LOG

    # success
    if [ "$1" == "3" ]; then
        cleanup $1 "$2"
        exit 0
    fi

    # failure
    if [ "$1" == "0" ]; then
        cleanup $1 "$2"
        exit 1
    fi
}

echo "******* New agent_install.sh execution with $1 $2 at `date` *******" >> $DEBUG_LOG
# clean old stuff
$DBSET installer:agent_install_completed -1
$DBSET installer:agent_install_msg

if [ "X$1" == "X" ]; then
	echo "Missing DA package"
	exit 1
fi

# according to ns_action::Action_Src 2 is WEB_UI source and ns_action::Action_Src 8 is WEB_UI2
if [ "X$2" == "X2" ] || [ "X$2" == "X8" ]; then
	IS_WEB_UI=1
	echo "This is a web-ui request" >> $DEBUG_LOG
fi

TGZ_FILE=$1
echo "TGZ_FILE=$TGZ_FILE" >> $DEBUG_LOG
TGZ_FILE_BASENAME=`basename $TGZ_FILE`
echo "TGZ_FILE_BASENAME=$TGZ_FILE_BASENAME" >> $DEBUG_LOG

echo "**** Extracting TGZ package..." | tee -a $DEBUG_LOG

echo "Files inside rpm:" >> $DEBUG_LOG
RPM_FILE=$( $GTAR_LIST $TGZ_FILE 2>&1 | tee -a $DEBUG_LOG  )

if [ $(echo $RPM_FILE | wc -w) != 1 ] || echo $RPM_FILE | grep -qsv '\.rpm$'
    then log_and_exit_if_finished 0 "Error: File is not a DA package."
fi

$GTAR_DECOMP $TGZ_FILE -C $TMP_FOLDER 2>&1 >> $DEBUG_LOG

[ ${PIPESTATUS[0]} != 0 ] && log_and_exit_if_finished 0 "Error: TGZ decompression failed."


echo "**** Extracting RPM package. This will stop DA service..." | tee -a $DEBUG_LOG
RPM_FILE=$TMP_FOLDER/$RPM_FILE
$RPM_DECOMP $RPM_FILE 2>&1 | tee -a $DEBUG_LOG

[ ${PIPESTATUS[0]} != 0 ] && log_and_exit_if_finished 0 "Error: RPM process failed."


echo "**** Removing RPM file..." >> $DEBUG_LOG

$RM $RPM_FILE 2>&1 | tee -a $DEBUG_LOG

echo "**** Starting DA Service..." | tee -a $DEBUG_LOG

echo "before dastart self_update_in_progress=`$DBGET installer:self_update_in_progress`" >> $DEBUG_LOG

$DADIR/bin/dastart 2>&1 >> $DEBUG_LOG

RC=${PIPESTATUS[0]}
if [ $RC != 0 ]; then
    # in case of failure with dastart execution no need for timeout.
    echo "dastart returned with code $RC" >> $DEBUG_LOG
    log_and_exit_if_finished 1 "Warning: Deployment Agent service did not start. Manually start the Deployment Agent service from the CLI."
else
    i=1

    PREV_PIDOF_DAS=`pidof DAService`
    echo "before polling -> pidof DAService=$PREV_PIDOF_DAS" >> $DEBUG_LOG

    # This parameter (self_update_in_progress) is set to 2 during pre-installation script.
    # When DAService is started it should reset this parameter so we polling this every second for changes
    DBGET_OUT=`$DBGET installer:self_update_in_progress`
    while [ "$DBGET_OUT" == "2" ]; do

        PIDOF_DAS=`pidof DAService`
        [ "$PIDOF_DAS" != "$PREV_PIDOF_DAS" ] && echo "i=$i -> pidof DAService=$PIDOF_DAS" >> $DEBUG_LOG
        PREV_PIDOF_DAS=$PIDOF_DAS
        # every 30 seconds try to start the service again if it even hasn't been started at all
        (( $i % 30 == 0 )) && [ "$PIDOF_DAS" == "" ] && $DADIR/bin/dastart 2>&1 >> $DEBUG_LOG

        sleep 1
        ((i++))

        # after timeout of 180 seconds exit with failure
        [ $i -gt 180 ] && log_and_exit_if_finished 0 "Error: Deployment Agent service did not start correctly. Contact Check Point support for more information"
        DBGET_OUT=`$DBGET installer:self_update_in_progress`
    done
    echo "after polling i=$i -> pidof DAService=`pidof DAService`" >> $DEBUG_LOG
fi


log_and_exit_if_finished 3 "Deployment Agent package - $TGZ_FILE_BASENAME - installed successfully."


