#!/usr/bin/cgisu -r

# Copyright Check Point

source subs.tcl
source debug.tcl
source showResult.tcl
source json.tcl

proc escape {input} {
	# this function is used to fix a special issue in tcl - when reading a string with the following format: <text>"<text> (any text, followed by " and then some other text, with no spaces or other special characters)
	return [string map {\" \\"} $input]
}
# " this comment just to set highlighting in notepad++

proc newline_to_br {input} {
	#translate \n into <br>
	return [string map {\\n <br>} $input]
}			

proc create_file {dst} {
	if {0 == [file exist $dst]} {
		set fp [open $dst {WRONLY CREAT} 0666]
		close $fp
	}
}

namespace eval ::installer {
	variable debug 0
	global dFile
	set dFile "/tmp/installer.post.debug"
	set trigger_file "/opt/CPda/bin/DAClient"
	set feedback_path "/var/log/CPda/tmp/"
	set feedback_prefix "CPUSE_Feedback_"
	set cp_install_log_path "/opt/CPInstLog/"
	set inconvertible_json_file_name "CPUSE_Inconvertible_JSON_Content"
	set src_flag "--src"
	set web_ui_src "WEB_UI"
	set flavor ""
	set role ""

	set err_list [list]
	set set_list [list]
	set result 0
	global method
	global myDb
	global bind_ping 
	set bind_ping "installer"
	global bind_volatile
	set bind_volatile "volatile"
	
	proc set_volatile {db key value} {
		libdb set $db "volatile:installer:$key" $value
	}
	
	proc get_volatile {db key} {
		return [libdb get $db "volatile:installer:$key"]
	}
	
	# messages can have special characters. If you use this messages - you need to escape them
	proc get_event_log_messages {} {
		set msgsList [list]
		if {[file isfile "/opt/CPInstLog/ui_messages_ver2"]} {
			set msg_file [open "/opt/CPInstLog/ui_messages_ver2"]
			set input [read $msg_file]
			close $msg_file
			set msgs [split $input "\n"]
			foreach cur_msg $msgs {
				set msgDict [dict create]
				set msg_data [split $cur_msg "|"]
				dict set msgDict notification_type [lindex $msg_data 0]
				dict set msgDict ballon_flag [lindex $msg_data 1]
				dict set msgDict time_stamp [lindex $msg_data 2]
				dict set msgDict message [lindex $msg_data 3]
				lappend msgsList $msgDict
			}
		}
		return $msgsList
	}	
	
	if {[catch {libdb init -local} myDb]} {
		#failed to initialize!!!
		ipso -nohtml
		MIME text
		errorOut "Unable to connect to database ${myDb}"
	}

	ipso -nohtml
	set method [get_Method]
	MIME text
	#printDebug

	set web_ui_version [libdb get $myDb "${bind_ping}:DA_GUI_version"]
	if {$web_ui_version == "WEB_UI2"} {
		set web_ui_src "WEB_UI2"
	}
	global Q_Names
	if {$method == "POST"} {
		set response ""
		foreach var $Q_Names {
			if {$var == "set_ftw_ifc"} {
				set web_ui_src "FTW_IFC"
			}
			if {$var == "flavor"} {
				set flavor [getVal $var]
			}
			if {$var == "role"} {
				set role [getVal $var]
			}
		}
		foreach var $Q_Names {
			set log ""
			set val [getVal $var]
			#set parts [split $var :]
			#set action [lindex $parts 0]

			if {$var == "mark_ftw"} {
				# mark the relevant package for the First Time Wizard
				libdb set $myDb "${bind_ping}:mark_ftw" $val
			}
			
			if {$var == "prepare_html_report_for_pkg"} {
				set html_res [catch {exec $trigger_file prepare_html_report $val} output]
				if {$html_res == 0} {
					set response "/html_reports/html_report.html"
				} else {
					set response ""
				}	
			}
			
			if {$var == "clean_upload_folder"} {
				file mkdir /var/log/upload
				foreach file [glob /var/log/upload/*] {
					file delete $file
				}
			}
			
			if {$var == "suppress_reboot"} {
				# set the suppress_reboot flag
				[set_volatile $myDb "suppress_reboot" 1]
			}

			if {$var == "download_and_install"} {
				# mark download + install
				libdb set $myDb "${bind_ping}:packages:${val}" 2
				libdb set $myDb "${bind_ping}:packages:${val}:download_and_install" 1
			}
			if {$var == "download_and_clean_install"} {
				# mark download + install
				libdb set $myDb "${bind_ping}:packages:${val}" 2
				libdb set $myDb "${bind_ping}:packages:${val}:download_and_install" 1
				libdb set $myDb "${bind_ping}:packages:${val}:download_and_install_role" $role
			}

			if {$var == "download_and_upgrade"} {
				# mark download + upgrade
				libdb set $myDb "${bind_ping}:packages:${val}" 2
				libdb set $myDb "${bind_ping}:packages:${val}:download_and_upgrade" 1
			}
			
			if {$var == "download"} {
				# do download
				libdb set $myDb "${bind_ping}:packages:${val}" 2
				#set response [libdb get $myDb "${bind_ping}:download_pkg:${val}"]
				set response [exec $trigger_file download $val $src_flag $web_ui_src]
			}
			
			if {$var == "pause"} {
				# do pause
				libdb set $myDb "${bind_ping}:packages:${val}:stop_download" 1
			}
			
			if {$var == "install"} {
				# do install
				set response [exec $trigger_file install $val $src_flag $web_ui_src]
			}

			if {$var == "clean_install"} {
				# do clean install
				if { $role == "" } {
					set response [exec $trigger_file clean_install $val $src_flag $web_ui_src]
				} else {
					set response [exec $trigger_file clean_install $val role $role $src_flag $web_ui_src]
				}
			}

			if {$var == "upgrade"} {
				# do install
				#set response [libdb get $myDb "${bind_ping}:install_pkg:${val}"]
				set response [exec $trigger_file upgrade $val $src_flag $web_ui_src]
			}	
			
			if {$var == "uninstall"} {
				# do uninstall
				#set response [libdb get $myDb "${bind_ping}:uninstall:${val}"]
				set response [exec $trigger_file uninstall $val $src_flag $web_ui_src]
			}	
			
			if {$var == "uninstall_completely"} {
				# do uninstall
				set response [exec $trigger_file uninstall_completely $val $src_flag $web_ui_src]
			}

			if {$var == "uninstall_last_take"} {
				# do uninstall
				set response [exec $trigger_file uninstall_last_take $val $src_flag $web_ui_src]
			}
			
			if {$var == "action"} {
				set response [libdb get $myDb "${bind_ping}:${val}"]
			}
			
			if {$var == "private_url"} {
				#set response [libdb get $myDb "${bind_ping}:add_url:${val}"]
				set response [exec $trigger_file add_private_package $val $src_flag $web_ui_src]
			}
			
			if {$var == "update_log_permissions_package"} {
				set pkg [getVal update_log_permissions_package]
				set type [getVal update_log_permissions_type]
				
				if { $pkg != "" } {
					if { $pkg == "FullEventLog" } {					
						set path "/opt/CPInstLog/DA_UI.log"
						file attributes $path -permissions 0664
					} else {
						if { $type != "" } {
							if { $type == "both" } {
								set path [libdb get $myDb "installer:packages:${pkg}:i_log_file"]
								file attributes $path -permissions 0664
								set path [libdb get $myDb "installer:packages:${pkg}:u_log_file"]
								file attributes $path -permissions 0664
							} else {						
								set path [libdb get $myDb "installer:packages:${pkg}:${type}_file"]
								file attributes $path -permissions 0664
							}
						}
					}
				}
			}
			
			if { $var == "self_update" } {
				libdb set $myDb "${bind_ping}:self_update_pending" ""
				#libdb set $myDb "${bind_ping}:do_self_update" $val
				set response [exec $trigger_file self_update]
			}
			
			if { $var == "upload" } {
				set response [exec $trigger_file import $val $src_flag $web_ui_src]
			}

			if { $var == "agent_install" } {
				set response [exec $trigger_file agent_install $val $src_flag $web_ui_src]
			}

			if { $var == "reboot" } {
				libdb set $myDb "${bind_ping}:reboot_pending" ""
				libdb set $myDb "${bind_ping}:do_reboot" $val
			}
			
			if { $var == "downtime_ack" } {
				libdb set $myDb "${bind_ping}:downtime_ack" 1
			}
			
			if { $var == "reboot_ack"} {
				libdb set $myDb "${bind_ping}:reboot_ack" ""
			}

			if { $var == "order_filter" } {
				libdb set $myDb "${bind_ping}:order_filter" ""
			}

			if { $var == "update_now" } {
				libdb set $myDb "${bind_ping}:connection_error_msg" ""
				#set response [libdb set $myDb "${bind_ping}:update_now" $val]
				set response [exec $trigger_file update $src_flag $web_ui_src]
			}

			if { $var == "export_pkg" } {
				#set response [libdb set $myDb "${bind_ping}:export_download" $val]
				set response [exec $trigger_file export $val $src_flag $web_ui_src]
			}
			
			if { $var == "exported_session_id" } {
				libdb set $myDb "${bind_ping}:export_session_id" $val
			}
			
			if { $var == "ftw_random"} {
				libdb set $myDb "${bind_ping}:ftw_random" $val
			}

			if { $var == "forced_reboot_ack"} {
				libdb set $myDb "${bind_ping}:forced_reboot_ack" ""
			}
		
			if { $var == "check_install_req" } {
				set response [libdb set $myDb "${bind_ping}:check_install_req" $val]
				if { $flavor == "" } {
					if { $role == "" } {
						set response [exec $trigger_file verify $val $src_flag $web_ui_src]
					} else {
						set response [exec $trigger_file verify $val role $role $src_flag $web_ui_src]
					}
				} else {
					if { $role == "" } {
						set response [exec $trigger_file verify $val flavor $flavor $src_flag $web_ui_src]
					} else {
						set response [exec $trigger_file verify $val flavor $flavor role $role $src_flag $web_ui_src]
					}
				}
			}
			
			if { $var == "delete_pkg"} {
				#set response [libdb set $myDb "${bind_ping}:delete_pkg" $val]
				set response [exec $trigger_file delete $val $src_flag $web_ui_src]
			}
			
			if { $var == "do_delayed_reboot"} {
				libdb set $myDb "${bind_ping}:do_delayed_reboot" $val
			}
			
			if { $var == "delay_reboot"} {
				libdb set $myDb "${bind_ping}:delay_reboot" $val
			}
			
			if { $var == "ignore_critical"} {
				libdb set $myDb "${bind_ping}:ignore_critical" $val
			}
			
			if { $var == "installed_hfs_ack"} {
				libdb set $myDb "${bind_ping}:installed_hfs:ack" $val
			}
			
			if { $var == "del_poll_parameter" } {
				libdb set $myDb "${bind_ping}:poll_parameter" ""
			}
			
			if { $var == "stop_download" } {
				libdb set $myDb "${bind_ping}:packages:${val}:stop_download" 1
			}
			
			if { $var == "action_resumed" } {
				libdb set $myDb "${bind_ping}:packages:${val}:action_resumed" 0
			}

			if { $var == "is_approved_user_send_report" } {
				libdb set $myDb "${bind_ping}:is_send_cloud_report" $val
			}


			if { $var == "is_approved_user_continue_install" } {
			   libdb set $myDb "${bind_ping}:is_approved_continue_install" $val
			}

			if { $var == "detect_last_version" } {
				libdb set $myDb "${bind_ping}:last_known_version" ""
				libdb set $myDb "${bind_ping}:last_known_version_filename" ""
				#set response [libdb set $myDb "${bind_ping}:detect_last_version" 1]
				set response [exec $trigger_file detect_last_version]
			}
			
			if { $var == "search_start" } {
				foreach search_res [libdb iterate children $myDb "${bind_ping}:search:result"] {
					libdb set $myDb "${bind_ping}:search:result:${search_res}" ""
				}
				set response [exec $trigger_file search_on_cloud $val]
				#no need to return the response.
				exit 0
				}
			if { $var == "reset_priv_pkg_enter_grid_status" } {
				libdb set $myDb "${bind_ping}:search:result:$val:enter_grid"
			}
			if { $var == "reset_search_res_db" } {
				foreach search_res [libdb iterate children $myDb "${bind_ping}:search:result"] {
					libdb set $myDb "${bind_ping}:search:result:${search_res}"							
				}
			}
			if { $var == "show_octo_promo" } {
			   libdb set $myDb "${bind_ping}:show_octo_promo" $val
			}
			if { $var == "collect_logs" } {
				libdb set $myDb "${bind_ping}:collect_logs" "1"
			}

			if {$var == "switch_gui"} {
				exec da_cli switch_gui source=$web_ui_src
			}
			
			if {$var == "dont_show_switch_gui"} {
				libdb set $myDb "${bind_ping}:show_switch_gui_pop" "0"
			}

			if {$var == "save_feedback"} {
				set feedback_file [ concat $feedback_path$feedback_prefix%%[ exec /usr/bin/date +_%s].txt ]
				create_file $feedback_file 
				set fh [open $feedback_file w]
				set rv ""
				regsub -all {[^[:print:][:blank:]\n]+} $val {} sanitized
				if {[catch {puts $fh $sanitized} res]} {
					error "$res"
				}
				close $fh
				return $rv
			}
			if {$var == "save_inconvertible_json"} {

				set inconvertible_json_content_txt_file [ concat $cp_install_log_path$inconvertible_json_file_name.txt ]
				if { ![file exists $inconvertible_json_content_txt_file] }
				{
					create_file $inconvertible_json_content_txt_file
					set rv ""
					set fh [open $inconvertible_json_content_txt_file w]
					if {[catch {puts $fh $val} res]} {
						error "$res"
					}
					close $fh
					return $rv
				}
			}
		}
		libdb set $myDb :save ""

		if { $log != "" } {
			HTML "$full_log"
		}
		
		if { $response != "" } {
			HTML "${response}"
		}
	}
	if {$method == "GET"} {
		set session_id ""
		set os_build ""
		set cur_version ""
		set search_packages 0
		set priv_pkg_name_val ""
		
		
		foreach var $Q_Names {
			set val [getVal $var]
			if { $var == "session_id" } {
				set session_id $val
			}
			if { $var == "search_polling" } {
				set search_packages 1
			}
			if { $var == "priv_pkg_name_val" } {
				set priv_pkg_name_val $val				
			}
			
			if {$var == "get_machine_details"} {
				set MachineDetails [dict create]
				#get gaia current take number
				set os_build_val [libdb get $myDb :version:os:build]
				dict set MachineDetails os_build $os_build_val
				#get gaia os version
				set cur_version_val [libdb get $myDb "snap:show:current:version"]
				dict set MachineDetails cur_version $cur_version_val
				
				HTML [::dict2json::compile_json dict $MachineDetails]
				libdb close $myDb
				exit 0
			}
			if {$var == "get_da_conf"} {
				HTML [exec da_cli edit_configuration operation=show_config source=$web_ui_src]
				libdb close $myDb
				exit 0
			}
			
			if {$var == "get_event_log_messages"} {
				set messages [get_event_log_messages]
				# escaping all occurrences of special characters
				set messages [string map {\" \\" \r\n \\r \n \\r "\\" ""} $messages] 
				dict set messagesDict msgsList $messages
				HTML [::dict2json::compile_json {dict msgsList {list {dict}}} $messagesDict]
				libdb close $myDb
				exit 0
			}
			
			if {$var == "get_upgrade_details"} {
				set UpgradeDetails [dict create]
				#get the size of parttition we should create for upgarde
				dict set UpgradeDetails new_partition_disk_size [libdb get $myDb "${bind_ping}:new_partition_disk_size"]
				#get the free unpartitioned disk space on machine
				dict set UpgradeDetails free_unpartitioned_disk_space [libdb get $myDb "${bind_ping}:free_unpartitioned_disk_space"]
				
				HTML [::dict2json::compile_json dict $UpgradeDetails]
				libdb close $myDb
				exit 0
			}
		}

		if { $search_packages } {
			dict set searchDict success true
			dict set rDict search [list]
			set finished 0
			
			foreach search_res [libdb iterate children $myDb "${bind_ping}:search:result"] {
				set ans ""
				set releaseDate ""
				set ans [libdb get $myDb "${bind_ping}:search:result:${search_res}"]
				set releaseDate [libdb get $myDb "${bind_ping}:search:result:${search_res}:releaseDate"]
				set fileSize [libdb get $myDb "${bind_ping}:search:result:${search_res}:fileSize"]

				if { $search_res == "match" && $ans == "finish" } {
					set finished 1
				} 			
					set jsDict [dict create]
					dict set jsDict js_text $search_res
					dict set jsDict js_desc $ans
					dict set jsDict js_id $search_res				        
					dict set jsDict js_releaseDate $releaseDate
					dict set jsDict js_fileSize $fileSize
						
					if { $search_res == "match" && $ans == "none" } {
						set finished 1
					}
					
						dict lappend rDict search $jsDict
				
			}
			
			if { !$finished } {
				dict set rDict search [list]
			}
			dict set searchDict data $rDict
			
			HTML [::dict2json::compile_json {dict data {dict search {list {dict}}} } $searchDict]
			libdb close $myDb
			exit 0
		}
		set searchResGridDict [dict create]
		if { $priv_pkg_name_val != "" } {
			set searchResEnterGridStatus [libdb get $myDb "${bind_ping}:search:result:${priv_pkg_name_val}:enter_grid"]			
			dict set searchResGridDict package_name $priv_pkg_name_val
			dict set searchResGridDict searchResEnterGridStatus $searchResEnterGridStatus	
		}
		set hideBrand [libdb get $myDb "${bind_ping}:hide_brand"]
		set isDArunning [libdb get $myDb "${bind_ping}:isDArunning"]
		set isDAdisabled [libdb get $myDb "${bind_ping}:disabled"]
		set connection_error_msg [libdb get $myDb "${bind_ping}:connection_error_msg"]
		set ftw_random [libdb get $myDb "${bind_ping}:ftw_random_res"]
		set da_updated [libdb get $myDb "${bind_ping}:da_updated"]
		set show_switch_gui_pop [libdb get $myDb "${bind_ping}:show_switch_gui_pop"]
		# indicates that self update is in progress: 1 if in progress none otherwise.
		set self_update_in_progress [libdb get $myDb "${bind_ping}:self_update_in_progress"]
		
		if { $da_updated != "" } {
			libdb set $myDb "${bind_ping}:da_updated"
		}
		set fcd_da [libdb get $myDb "${bind_ping}:fcd_da"]
		set packagesList [list]
		foreach package [libdb iterate children $myDb "${bind_ping}:packages"] {
			if {[libdb get $myDb "${bind_ping}:packages:${package}"] == ""} {
				continue ; # doesn't really exist; skip
			}
			if {[libdb get $myDb "${bind_ping}:packages:${package}:install_automatic"] == "1"} {
				continue ; # we don't want to show these
			}
			set is_out_of_enabled 1
			if {[libdb get $myDb "${bind_ping}:packages:${package}:progress:TotalSteps"]=="0" || [libdb get $myDb "${bind_ping}:packages:${package}:progress:TotalSteps"]==""} {
				set is_out_of_enabled 0
			}
			set is_show_progress_enabled 1
			if {[libdb get $myDb "${bind_ping}:packages:${package}:progress:CompletedSteps"]=="" && [libdb get $myDb "${bind_ping}:packages:${package}:progress:Description"]==""} {
							set is_show_progress_enabled 0
			}
			set show_progress_dict [dict create \
									total_steps [libdb get $myDb "${bind_ping}:packages:${package}:progress:TotalSteps"] \
									completed_steps [libdb get $myDb "${bind_ping}:packages:${package}:progress:CompletedSteps"] \
									description [libdb get $myDb "${bind_ping}:packages:${package}:progress:Description"]\
									is_out_of_enabled $is_out_of_enabled \
									is_show_progress_enabled $is_show_progress_enabled]
			set pkgDict [dict create]
			dict set pkgDict package_name $package
			dict set pkgDict status [libdb get $myDb "${bind_ping}:packages:${package}"]
			dict set pkgDict status_text [libdb get $myDb "${bind_ping}:packages:${package}:status_text"]
			dict set pkgDict status_progress_gif [libdb get $myDb "${bind_ping}:packages:${package}:status_progress_gif"]
			dict set pkgDict progress [libdb get $myDb "${bind_ping}:packages:${package}:progress"]
			dict set pkgDict cur_size [libdb get $myDb "${bind_ping}:packages:${package}:progress:csize"]
			dict set pkgDict cur_rate [libdb get $myDb "${bind_ping}:packages:${package}:progress:cur_rate"]
			dict set pkgDict remaining_time [libdb get $myDb "${bind_ping}:packages:${package}:progress:remaining_time"]
			dict set pkgDict download_retry_counter [libdb get $myDb "${bind_ping}:packages:${package}:progress:download_retry_counter"]
			dict set pkgDict pkgkey [escape [libdb get $myDb "${bind_ping}:packages:${package}:pkgkey"]]
			dict set pkgDict size [libdb get $myDb "${bind_ping}:packages:${package}:size"]
			dict set pkgDict show_progress $show_progress_dict
			dict set pkgDict blink_roles [libdb get $myDb "${bind_ping}:packages:${package}:blink_property:blink_roles"]
			dict set pkgDict blink_roles_cli [libdb get $myDb "${bind_ping}:packages:${package}:blink_property:blink_roles_cli"]
			set is_hfa [libdb get $myDb "${bind_ping}:packages:${package}:is_hfa"]
			dict set pkgDict is_hfa $is_hfa
			set pkg_type [libdb get $myDb "${bind_ping}:packages:${package}:pkg_type"]
			dict set pkgDict pkg_type $pkg_type
			dict set pkgDict pkg_type_modified "$pkg_type"

			# discard previous value
			set major_display_name ""
			if {$pkg_type == "4"} {
				if {$is_hfa != "1"} {
					dict set pkgDict pkg_type_modified "1"
				}
			}
			if {$pkg_type == "5"} {
				set is_blink [libdb get $myDb "${bind_ping}:packages:${package}:is_blink_image"]
				if {$is_blink == "1"} {
					dict set pkgDict pkg_type_modified "11"
					set major_display_name [libdb get $myDb "${bind_ping}:packages:${package}:major_display_name"]
				}
			}

			set ptm [dict get $pkgDict pkg_type_modified]
			if {$major_display_name != ""} {
				dict set pkgDict group_by "$ptm#${major_display_name}"
			} else {
				dict set pkgDict group_by "$ptm"
			}

			set i_log_file [libdb get $myDb "${bind_ping}:packages:${package}:i_log_file"]
			dict set pkgDict i_log_file $i_log_file
			if {![file isfile "${i_log_file}"]} {
				dict set pkgDict i_log_file ""
			}
			set u_log_file [libdb get $myDb "${bind_ping}:packages:${package}:u_log_file"]
			dict set pkgDict u_log_file $u_log_file
			if {![file isfile "${u_log_file}"]} {
				dict set pkgDict u_log_file ""
			}

			set version_name [libdb get $myDb "${bind_ping}:packages:${package}:major_display_name"]
			set version_name [regsub ".00" $version_name ""]
			if {$version_name != ""} {
				set result [catch {open /tmp/fullha_script w} fd]
				puts $fd "#!/bin/bash"
				puts $fd ". /opt/CPshared/5.0/tmp/.CPprofile.sh"
				puts $fd "\$CPDIR/bin/cpprod_util CPPROD_GetValue CPupgrade-tools-${version_name} BuildNumber 1"
				puts $fd "rm /tmp/fullha_script"
				puts $fd "exit 0"
				close ${fd}
				file attributes /tmp/fullha_script -permissions 777
				set tmpStr ""
				set result [catch {exec  /tmp/fullha_script} tmpStr]
				dict set pkgDict upgrade_tools_build $tmpStr
			}

			dict set pkgDict upgrade_tools_active [libdb get $myDb "${bind_ping}:packages:${package}:upgrade_tools_active"]
			dict set pkgDict available_from [libdb get $myDb "${bind_ping}:packages:${package}:available_from"]
			dict set pkgDict origin [libdb get $myDb "${bind_ping}:packages:${package}:origin"]
			dict set pkgDict added_on [libdb get $myDb "${bind_ping}:packages:${package}:added_on"]
			dict set pkgDict downloaded_on [libdb get $myDb "${bind_ping}:packages:${package}:downloaded_on"]
			dict set pkgDict imported_on [libdb get $myDb "${bind_ping}:packages:${package}:imported_on"]
			dict set pkgDict installed_on [libdb get $myDb "${bind_ping}:packages:${package}:installed_on"]
			dict set pkgDict restored_on [libdb get $myDb "${bind_ping}:packages:${package}:restored_on"]
			dict set pkgDict description [escape [libdb get $myDb "${bind_ping}:packages:${package}:description"]]
			dict set pkgDict dc_description [escape [libdb get $myDb "${bind_ping}:packages:${package}:dc_description"]]
			dict set pkgDict chk_description [escape [libdb get $myDb "${bind_ping}:packages:${package}:chk_description"]]
			dict set pkgDict install_description [escape [libdb get $myDb "${bind_ping}:packages:${package}:install_description"]]
			if {$fcd_da == ""} {
			   dict set pkgDict self_update [libdb get $myDb "${bind_ping}:packages:${package}:install_critical"]
			}
			dict set pkgDict additional_info [libdb get $myDb "${bind_ping}:packages:${package}:additional_info_from_last_action"]
			dict set pkgDict d_time_sec [libdb get $myDb "${bind_ping}:packages:${package}:d_time:usec"]
			dict set pkgDict d_time_usec [libdb get $myDb "${bind_ping}:packages:${package}:d_time:sec"]
			dict set pkgDict d_failed [escape [libdb get $myDb "${bind_ping}:packages:${package}:d_failed"]]
			dict set pkgDict failed [escape [libdb get $myDb "${bind_ping}:packages:${package}:failed"]]
			dict set pkgDict d_failed_time [libdb get $myDb "${bind_ping}:packages:${package}:d_failed:time"]
			dict set pkgDict failed_time [libdb get $myDb "${bind_ping}:packages:${package}:failed:time"]
			dict set pkgDict bundle_err [escape [libdb get $myDb "${bind_ping}:packages:${package}:bundle_err"]]
			dict set pkgDict disable_upgrade [libdb get $myDb "${bind_ping}:packages:${package}:disable_upgrade"]
			dict set pkgDict order [escape [libdb get $myDb "${bind_ping}:packages:${package}:order"]]
			dict set pkgDict check_install_res [libdb get $myDb "${bind_ping}:packages:${package}:check_install_res"]
			dict set pkgDict check_install_success [libdb get $myDb "${bind_ping}:packages:${package}:check_install_success"]
			dict set pkgDict self_test_res [libdb get $myDb "${bind_ping}:packages:${package}:self_test_res"]
			dict set pkgDict old_state [libdb get $myDb "${bind_ping}:packages:${package}:old_state"]
			dict set pkgDict brought_version [libdb get $myDb "${bind_ping}:packages:${package}:brought_version"]
			dict set pkgDict display_name [escape [libdb get $myDb "${bind_ping}:packages:${package}:display_name"]]
			dict set pkgDict has_metadata [libdb get $myDb "${bind_ping}:packages:${package}:has_metadata"]
			dict set pkgDict category [libdb get $myDb "${bind_ping}:packages:${package}:category"]
			dict set pkgDict is_pkg_in_repository [libdb get $myDb "${bind_ping}:packages:${package}:is_pkg_in_repository"]
			dict set pkgDict action_resumed [libdb get $myDb "${bind_ping}:packages:${package}:action_resumed"]
			dict set pkgDict legacy_hotfix [libdb get $myDb "${bind_ping}:packages:${package}:legacy_hotfix"]
			dict set pkgDict product [libdb get $myDb "${bind_ping}:packages:${package}:product"]
			dict set pkgDict bundle_contains_os [libdb get $myDb "${bind_ping}:packages:${package}:bundle_contains_os"]
			dict set pkgDict requires_reboot [libdb get $myDb "${bind_ping}:packages:${package}:requires_reboot"]
			dict set pkgDict is_uninstall_options [libdb get $myDb "${bind_ping}:packages:${package}:is_uninstall_options"]
			dict set pkgDict is_backup_completely [libdb get $myDb "${bind_ping}:packages:${package}:is_backup_completely"]
			dict set pkgDict is_backup_last_take [libdb get $myDb "${bind_ping}:packages:${package}:is_backup_last_take"]
			dict set pkgDict previous_installed_take [libdb get $myDb "${bind_ping}:packages:${package}:previous_installed_take"]
			dict set pkgDict requires_cpstop_cpstart [libdb get $myDb "${bind_ping}:packages:${package}:requires_cpstop_cpstart"]
			dict set pkgDict exist_html_report [libdb get $myDb "${bind_ping}:packages:${package}:exist_html_report"]
			dict set pkgDict part_of_major [libdb get $myDb "${bind_ping}:packages:${package}:part_of_major"]
			dict set pkgDict pending_reboot [get_volatile $myDb "packages:${package}:pending_reboot"]
			dict set pkgDict block_uninstall [libdb get $myDb "${bind_ping}:packages:${package}:block_uninstall"]
			dict set pkgDict sort_group [libdb get $myDb "${bind_ping}:packages:${package}:sort_group"]
			dict set pkgDict sort_index [libdb get $myDb "${bind_ping}:packages:${package}:sort_index"]
			dict set pkgDict patching_failure_detected [libdb get $myDb "${bind_ping}:packages:${package}:patching_failure_detected"]
			
			set package_parent [libdb get $myDb "${bind_ping}:packages:${package}:parent"]
			dict set pkgDict package_parent $package_parent
			dict set pkgDict package_child [libdb get $myDb "${bind_ping}:packages:${package}:child"]
			dict set pkgDict package_conflict [libdb get $myDb "${bind_ping}:packages:${package}:conflict"]
			dict set pkgDict installed_deps [libdb get $myDb "${bind_ping}:packages:${package}:install_deps"]
			dict set pkgDict uninstalled_deps [libdb get $myDb "${bind_ping}:packages:${package}:uninstall_deps"]
			dict set pkgDict verification_response [escape [newline_to_br [libdb get $myDb "${bind_ping}:packages:${package}:verification_response"]]]
			set available_from_sort [libdb get $myDb "${bind_ping}:packages:${package}:available_from_sort"]
			dict set pkgDict available_from_sort $available_from_sort
			if {$available_from_sort == "" } {
				if {$package_parent != "" } {
					# if the package is legacy and has parents - show it above other legacy packages (REMOVE IN MS3)
					dict set pkgDict available_from_sort 1111-1-1
				}
			}

			set d_complete [libdb get $myDb "${bind_ping}:packages:${package}:d_complete"]
			dict set pkgDict d_complete $d_complete
			if { $d_complete != "" } {
				libdb set $myDb "${bind_ping}:packages:${package}:d_complete" ""
			}
			
			set i_complete [libdb get $myDb "${bind_ping}:packages:${package}:i_complete"]
			dict set pkgDict i_complete $i_complete
			if { $i_complete != "" } {
				libdb set $myDb "${bind_ping}:packages:${package}:i_complete" ""
			}
			
			set u_complete [libdb get $myDb "${bind_ping}:packages:${package}:u_complete"]
			dict set pkgDict u_complete $u_complete
			if { $u_complete != "" } {
				libdb set $myDb "${bind_ping}:packages:${package}:u_complete" ""
			}
			
			set tagsDict [dict create]
			foreach package_tag_header [libdb iterate children $myDb "${bind_ping}:packages:${package}:tag"] {
				if { $package_tag_header != "" } {
					dict set tagsDict $package_tag_header [libdb get $myDb "${bind_ping}:packages:${package}:tag:${package_tag_header}"]
				}
			}
			dict set pkgDict package_tags $tagsDict
			
			lappend packagesList $pkgDict
		
		}
		
		# for the popup messages - we need to send the messages every call
		set msgsList [get_event_log_messages]
		
		set installed_hfs [libdb get $myDb "${bind_ping}:installed_hfs"]

		set self_update [libdb get $myDb "${bind_ping}:self_update_pending"]
		set exported_session_id  [libdb get $myDb "${bind_ping}:export_session_id"]
		set export_pkg_error [libdb get $myDb "${bind_ping}:export_pkg_error"]
		if { $export_pkg_error != "" } {
			if {$session_id == $exported_session_id } {
				libdb set $myDb "${bind_ping}:export_pkg_error" ""
			}
		}
		set su_timeout [libdb get $myDb "${bind_ping}:self_update_timeout"]
		set reboot [libdb get $myDb "${bind_ping}:reboot_pending"]
		set reboot_timeout [libdb get $myDb "${bind_ping}:reboot_timeout"]
		set downtime [libdb get $myDb "${bind_ping}:downtime"]
		set remove_su_popup [libdb get $myDb "${bind_ping}:remove_su_popup"]
		if { $remove_su_popup != "" } {
			libdb set $myDb "${bind_ping}:remove_su_popup" ""
		}
		set reboot_forced [libdb get $myDb "${bind_ping}:reboot_forced"]
		set check_install_res [libdb get $myDb "${bind_ping}:check_install_res"]
		if { $check_install_res != "" } {
			libdb set $myDb "${bind_ping}:check_install_res" ""
		}
		set check_install_filename [libdb get $myDb "${bind_ping}:check_install_filename"]

		set check_install_details [libdb get $myDb "${bind_ping}:check_install_details"]
		
		set installed_jumbo_take [libdb get $myDb "${bind_ping}:installed_jumbo_take"]
		
		set ignore_critical [libdb get $myDb "${bind_ping}:ignore_critical"]

		set pnp_license_exp [libdb get $myDb "${bind_ping}:pnp_license_exp"]

		set uda_disallowed [libdb get $myDb "${bind_ping}:uda_disallowed"]
	
		set poll_parameter [libdb get $myDb "${bind_ping}:poll_parameter"]
		
		set mark_ftw [libdb get $myDb "${bind_ping}:mark_ftw"]
		
		set delay_reboot_timeout [libdb get $myDb "${bind_ping}:delay_reboot_timeout"]
		
		set no_da_actions_allowed [get_volatile $myDb "no_da_actions_allowed"]

		set pkg_run_complete [libdb get $myDb "${bind_ping}:pkg_run_complete"]
		if { $pkg_run_complete != "" } {
			libdb set $myDb "${bind_ping}:pkg_run_complete" ""
		}
		
		set export_download [libdb get $myDb "${bind_ping}:export_pkg"]
		if { $export_download != "" } {
			if { $session_id == $exported_session_id } {
				libdb set $myDb "${bind_ping}:export_pkg" ""
			}
			if { $session_id != $exported_session_id } {
				set export_download ""
			}
		}
		set download_smartconsole [libdb get $myDb "${bind_ping}:download_smartconsole"]
		if { $download_smartconsole != "" } {
			libdb set $myDb "${bind_ping}:download_smartconsole" ""
		}
		
		set last_known_version [libdb get $myDb "${bind_ping}:last_known_version"]
		set last_known_version_filename [libdb get $myDb "${bind_ping}:last_known_version_filename"]
		
		set imported_package [libdb get $myDb "${bind_ping}:imported_package"]
		
		set current_version [libdb get $myDb "${bind_ping}:current_version"]
		
		set pkg_run_started [libdb get $myDb "${bind_ping}:pkg_run_started"]

		set mgmt_upgrade [libdb get $myDb "${bind_ping}:mgmt_upgrade"]
		set da_build [libdb get $myDb "${bind_ping}:da_build"]
		set product_type [libdb get $myDb ":version:product_type"]
		
		#get DA update status
		set update_status [libdb get $myDb "${bind_ping}:update_status"]	
		set update_status_message [libdb get $myDb "${bind_ping}:update_status_message"]
		set show_octo_promo [libdb get $myDb "${bind_ping}:show_octo_promo"]
		set last_update_time [libdb get $myDb "${bind_ping}:last_update_time"]
		set is_send_cloud_report [libdb get $myDb "${bind_ping}:is_send_cloud_report"]

		set is_approved_continue_install [libdb get $myDb "${bind_ping}:is_approved_continue_install"]

		set ddr_warning_msg [libdb get $myDb "${bind_ping}:ddr_warning_msg"]

		set collect_logs [libdb get $myDb "${bind_ping}:collect_logs"]
		if { $collect_logs == "1" } {
			exec da_cli collect_logs destination=/var/log/download/DA_logs.tgz source=$web_ui_src
			libdb set $myDb "${bind_ping}:collect_logs"
		}
		set alignDict [dict create]
		foreach category_entry [libdb iterate children $myDb "${bind_ping}:category_is_aligned"] {
			if { $category_entry != "" } {
				dict set alignDict $category_entry [libdb get $myDb "${bind_ping}:category_is_aligned:${category_entry}"]
			}
		}



		set dataDict [dict create \
			packages $packagesList \
			messages $msgsList \
			categoryDict $alignDict \
			searchResGridDict $searchResGridDict \
		]
		

			  
		set mainDict [dict create \
				success true \
				data $dataDict\
				hideBrand $hideBrand \
				isDArunning $isDArunning \
				isDAdisabled $isDAdisabled \
				self_update $self_update \
				collect_logs $collect_logs \
				su_timeout $su_timeout \
				reboot $reboot \
				reboot_timeout $reboot_timeout \
				da_updated $da_updated \
				show_switch_gui_pop $show_switch_gui_pop \
				self_update_in_progress $self_update_in_progress \
				connection_error_msg $connection_error_msg\
				downtime $downtime \
				remove_su_popup $remove_su_popup \
				reboot_forced $reboot_forced \
				check_install_res $check_install_res \
				installed_jumbo_take $installed_jumbo_take \
				check_install_filename $check_install_filename \
				check_install_details $check_install_details \
				ignore_critical $ignore_critical \
				pnp_license_exp $pnp_license_exp \
				uda_disallowed $uda_disallowed \
				poll_parameter $poll_parameter \
				pkg_run_complete $pkg_run_complete \
				ftw_random $ftw_random \
				export_download $export_download \
				export_pkg_error $export_pkg_error \
				delay_reboot_timeout $delay_reboot_timeout \
				no_da_actions_allowed $no_da_actions_allowed \
				mgmt_upgrade $mgmt_upgrade \
				installed_hfs $installed_hfs \
				pkg_run_started $pkg_run_started \
				da_build $da_build \
				product_type $product_type \
				download_smartconsole $download_smartconsole \
				exported_session_id $exported_session_id \
				mark_ftw $mark_ftw \
				current_version $current_version \
				imported_package $imported_package \
				last_known_version $last_known_version \
				last_known_version_filename $last_known_version_filename \
				update_status $update_status \
				update_status_message $update_status_message \
				show_octo_promo $show_octo_promo \
				last_update_time $last_update_time\
				is_send_cloud_report $is_send_cloud_report\
				is_approved_continue_install $is_approved_continue_install\
				ddr_warning_msg $ddr_warning_msg\
		]
		
		# escaping all occurances of special characters
		set mainDict [string map {\" \\" \r\n \\r \n \\r "\\" ""} $mainDict]

		
		HTML [::dict2json::compile_json \
			{dict data {dict packages {list {dict package_tags dict show_progress dict} } messages {list dict} categoryDict {dict} searchResGridDict {dict} } } $mainDict]
	
	}
	libdb close $myDb

}; # namespace

exit 0
